from django.conf.urls import include, url
from django.contrib import admin

from awl.rankedmodel import urls as ranked_urls

urlpatterns = [
    url(r'admin/', include(admin.site.urls)),

    url(r'rankedmodel/', include(ranked_urls)),
]



# awl.tests.test_utils.py
import sys
from django.test import TestCase
from django.http import HttpResponse

from six import StringIO

from awl.tests.models import Link
from awl.utils import (URLTree, refetch, refetch_for_update, render_page,
    render_page_to_string)
from awl.waelsteng import FakeRequest

# ============================================================================

class UtilsTest(TestCase):
    def test_url_tree(self):
        # print_tree() exercises everything, so run it and capture stdout
        tree = URLTree()

        saved_stdout = sys.stderr
        try:
            out = StringIO()
            sys.stdout = out
            tree.print_tree()

            # check for some of our rankedmodel items in the output
            output = out.getvalue().split('\n')
            self.assertIn('rankedmodel/', output)
            self.assertIn(('rankedmodel/move/(\d+)/(\d+)/(\d+)/$, ' 
                'name=awl-rankedmodel-move'), output)

        finally:
            sys.stdout = saved_stdout

    def test_refetch(self):
        link = Link.objects.create(url='url', text='text')
        link.text = 'foo'

        link = refetch(link)
        self.assertEqual('url', link.url)
        self.assertEqual('text', link.text)

        link.text = 'foo'
        link = refetch_for_update(link)
        self.assertEqual('url', link.url)
        self.assertEqual('text', link.text)

    def test_renders(self):
        request = FakeRequest()
        expected = 'Hello World\n'

        result = render_page_to_string(request, 'sample.html', {'name':'World'})
        self.assertEqual(expected, result)

        response = render_page(request, 'sample.html', {'name':'World'})
        self.assertEqual(expected, response.content.decode('ascii'))



# awl.tests.test_models.py
from django.test import TestCase

from awl.models import Counter, Lock
from awl.utils import refetch
from awl.tests.models import Validator

# ============================================================================

class ModelsTest(TestCase):
    def test_validating_mixin(self):
        # Validator overrides full_clean() to increment its counter,
        # full_clean() is called on every save, so after creation and save the
        # value should be 2
        v = Validator.objects.create()
        v.save()
        v = refetch(v)
        self.assertEqual(2, v.counter)

    def test_counter(self):
        count = Counter.objects.create(name='foo')
        Counter.increment('foo')
        count = refetch(count)
        self.assertEqual(1, count.value)

    def test_lock(self):
        # not much to test here except that it doesn't blow up
        Lock.objects.create(name='foo')
        Lock.lock_until_commit('foo')



from django.db import models, transaction

# ============================================================================
# Abstract Models
# ============================================================================

class TimeTrackModel(models.Model):
    """Abstract model with auto-updating create & update timestamp fields.

    :param created:
        Date/time when the model was created
    :param updated:
        Date/time when the model was last updated
    """
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class ValidatingMixin(models.Model):
    """Include this mixin to force model validation to happen on each
    ``save`` call.
    """
    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.full_clean()
        super(ValidatingMixin, self).save(*args, **kwargs)

# ============================================================================
# Concrete Models
# ============================================================================

class Counter(TimeTrackModel):
    """A named counter in the database with atomic update."""
    name = models.CharField(max_length=30)
    value = models.BigIntegerField(default=0)

    @classmethod
    def increment(cls, name):
        """Call this method to increment the named counter.  This is atomic on
        the database.

        :param name:
            Name for a previously created ``Counter`` object 
        """
        with transaction.atomic():
            counter = Counter.objects.select_for_update().get(name=name)
            counter.value += 1
            counter.save()

        return counter.value


class Lock(TimeTrackModel):
    """Implements a simple global locking mechanism across database accessors
    by using the ``select_for_update()`` feature.  Example:

    .. code-block:: python

        # run once, or use a fixture
        Lock.objects.create(name='everything')

        # views.py
        def something(request):
            Lock.lock_until_commit('everything')

    """
    name = models.CharField(max_length=30)

    @classmethod
    def lock_until_commit(cls, name):
        """Grabs this lock and holds it (using ``select_for_update()``) until
        the next commit is done.

        :param name:
            Name for a previously created ``Lock`` object 
        """
        Lock.objects.select_for_update().get(name=name)



import os

VERSION='0.6.1'

readme = os.path.join(os.path.dirname(__file__), 'README.rst')
long_description = open(readme).read()


SETUP_ARGS = dict(
    name='django-awl',
    version=VERSION,
    description='Miscellaneous django tools',
    long_description=long_description,
    url='https://github.com/cltrudeau/django-awl',
    author='Christopher Trudeau',
    author_email='ctrudeau+pypi@arsensa.com',
    license='MIT',
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='django,state machine',
    test_suite="load_tests.get_suite",
    install_requires=[
        'Django>=1.8',
        'wrench>=0.5.1',
    ],
    tests_require=[
        'mock>=1.3.0',
    ]
)

if __name__ == '__main__':
    from setuptools import setup, find_packages

    SETUP_ARGS['packages'] = find_packages()
    setup(**SETUP_ARGS)
